/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Thread message data wrapper class. Provides an easy interface to the thread message data.
 *
 * @class ?
 */
var glThreadDetails = glBase.extend({
	
	// Properties {
	
	/** @property {private glGmail} ? The gmail object for the page */
	_gmail: null,
	
	/** @property {String} ? The id of the thread these messages belong to */
	threadId: null,
	
	/** @property {glThreadMessage[]} ? */
	messages: null,
	
	/** @property {glRequest} ? */
	_request: null,
		
	// }
	
    /**
     * The constructor.
     * @constructor ?
	 * @param {glGmail} gmail - The gmail object for the page
     */
    constructor: function(gmail) {
		this._gmail = gmail;
		this.messages = [];
    },

	/**
	* Load a specific thread id
	* @function ?
	* @param {String} threadId - The id of the thread to get details for
	* @param {optional glDataObject} dataObject - Optional. Data object to use instead of a ajax request.
	*/
	load: function(threadId, dataObject){
		
		this._threadId = threadId;
		
        this._request = new glRequest(this._gmail, { 'view': 'cv', 'th':this._threadId, 'prf':'1', 'rt':'j', 'search':'query', 'q':'' });
        this._request.addEventListener(glRequest.COMPLETED, this._requestCallback, this);
		if( dataObject ){
			this.messages = dataObject.getThreadMessages();
		}else{
        	this._request.run();
		}

		// TODO decide if want to listen for notification of new 'ms' data items from TrafficWatcher; that could signify updated messages. Costly and not sure of value (as right now, we reload the messages of every viewing; and only view 1 thread at once). There is corresponding TODO in glTrafficWatcher
	},
	
	_requestCallback: function(){
		
		this.messages = this._request.data.getThreadMessages();
        this.dispatchEvent({name: glThreadDetails.LOADED});
	},
	
	
},{
    // Static Properties {
    
	LOADED: 'loaded',

    //}
});
glThreadDetails.implement(glEventDispatcher); // Add event dispatching support