/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Thread message data wrapper class. Provides an easy interface to the thread message data.
 *
 * @class ?
 */
var glThreadMessage = glBase.extend({
	
	// Properties {
	
	/** @property {private glGmail} ? The gmail object for the page */
	_gmail: null,
	
	
	/** @property {String} ? The id of this message */
	messageId: null,
	
	/** @property {int} ? The epoch formatted timestamp for the message */
	timestamp: null,
	
	/** @property {String} ? The message */
	message: null,
	
	/** @property {String} ? The date of the message */
	date: null,
	
	/** @property {Object} ? The sender. Format: {name:, email:} */
	sender: null,
	
	/** @property {String[]} ? The recipients. Format: [{name:, email:}] */
	recipients: null,
	
	/** @property {Object[]} ? Files. Format: [{attid:, name:, bytes:, icon:, url:, type:, preview:}] */
	files: null,
		
	// }
	
    /**
     * The constructor.
     * @constructor ?
	 * @param {glGmail} gmail
     */
    constructor: function(gmail) {
		this._gmail = gmail;
    },
	
},{
    // Static Methods {
    /**
     * Creates and returns a glThreadMessage object populated from the given data.
     * The reason it is passed a pre-parsed object from {@link glDataObject} is
     * that I'd like to keep most of the code that can change in a few places (the
     * UI widgets being an exception for the sake of code beauty and readability).
     * I'm not really attached to that decision though, so the parsing may be
     * moved into here.
     * @function {static glThreadMessage} ?
     * @param {Object} threadMessageData - Object containing exctracted thread data
	 * @param {glGmail} gmail - 
     * @return The created glThreadMessage obect
     */
    createFromData: function(threadMessageData, gmail) {
        var threadMessage = new glThreadMessage(gmail);

        // Poplulate created thread object
		for( p in threadMessageData ){
			threadMessage[p] = threadMessageData[p];
		}

        return threadMessage;
    },
    //}
});