/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The message compose view abstraction layer.
 *
 * @class ? @extends glUIView
 */
var glComposeView = glUIView.extend({
    // Properties {

		/** @property {Element} ? The send button in the top button row */
	    topSendButton: null,

	    /** @property {Element} ? The send button in the bottom button row */
	    bottomSendButton: null,

	    /** @property {Element} ? The tr containing "from" information */
	    fromRow: null,

	    /** @property {Element} ? The tr containing "to" information */
	    toRow: null,

	    /** @property {Element} ? The input element for the subject of the e-mail */
	    subjectField: null,

	    /** @property {Element} ? The textarea element for the recipient(s) */
	    toField: null,

	    /** @property {Element} ? The table element containing all compose fields */
	    composeTable: null,

	    /** @property {Element} ? The select element containing from entries */
	    fromSelect: null,

	    /** @property {Element} ? The textarea body when in plain text mode */
	    bodyPlain: null,

	    /** @property {Element} ? The iframe body when in rich text mode */
	    bodyRich: null,

	
	

    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {glGmail} gmail - The glGmail object for the page
     */
    connect: function(gmail) {
        glLogger.log('Connecting glComposeView object to view', 'glComposeView');

        this.base(gmail);


	    var doc = gmail.canvasDocument;

		var result = doc.evaluate("//div[contains(@class,'glViewRoot')]//textarea[@class='dV']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		if( !result.singleNodeValue ) result = doc.evaluate("//div[contains(@class,'glViewRoot')]//textarea[@class='iE5Yyc']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		this.bodyPlain = result.singleNodeValue;

		var w = gmail.canvasDocument.defaultView;
		for( var i = 0; i < w.frames.length; i++ ){
			if( w.frames[i].document.body.className.indexOf("editable")>-1 ){
				this.bodyRich = w.frames[i].document.body;
				break;
			}
		}

		var result = doc.evaluate("//div[contains(@class,'glViewRoot')]//select[@name='from']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		this.fromSelect = result.singleNodeValue;

		var result = doc.evaluate("//div[contains(@class,'glViewRoot')]//textarea[@name='to']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		this.toField = result.singleNodeValue;

		var result = doc.evaluate("//div[contains(@class,'glViewRoot')]//input[@name='subject']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		this.subjectField = result.singleNodeValue;

		var result = doc.evaluate("//div[contains(@class,'glViewRoot')]//table[contains(@class,'cf eA')]", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		if( !result.singleNodeValue ) result = doc.evaluate("//div[@class='uQLZXb']//form//table[contains(@class,'wHcYVb')]", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		this.composeTable = result.singleNodeValue;

		var result = doc.evaluate("//div[contains(@class,'glViewRoot')]//div[starts-with(@class,'eh')]//div[@role='button'][1]", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		if( !result.singleNodeValue ) result = doc.evaluate("//div[starts-with(@class,'LlWyA')]//div[@role='button'][1]", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		this.topSendButton = result.singleNodeValue;

		var result = doc.evaluate("//div[contains(@class,'glViewRoot')]//div[starts-with(@class,'CoUvaf')]//div[@role='button'][1]", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		this.bottomSendButton = result.singleNodeValue;


		
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting glComposeView from view', 'glComposeView');

		    this.topSendButton = null;
	        this.bottomSendButton = null;
	        this.fromRow = null;
	        this.toRow = null;
	        this.subjectField = null;
	        this.bodyPlain = null;
	        this.bodyRich = null;
	        this.composeTable = null;
	        this.toField = null;
	        this.fromSelect = null;

        this.base();
    },
    //}
});
