/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Listen to clicks to load Explorer Popup
 *
 * @class gtdEnvironment @extends glBase
 */
var gtdEnvironment = glBase.extend({
    // Properties {
    /**
     * The gtdMainPagePlugin object for the page
     * @property {gtdMainPagePlugin} ?
     */
    _gtd: null,

	/** @property {glEventsContainer} ? */
	_eventsContainer: null,
	
	
	/** @property {private Boolean[]} ? The rows checked before we kick in */
	_lastCheckedRows: null,
	
	
	/** @property {private glThread} */
	_currentThread: null,
	
    //}

    /**
     * The constructor.
     * @constructor ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    constructor: function(gtd) {
        this.base(gtd.gmail);
        this._gtd = gtd;
		this._eventsContainer = new glEventsContainer(this);
		this._attachedListeners = {};
		
		// Add click handler
		this._attachListeners();
		
		// Listen for view changes:
		this._gtd.gmail.page.addEventListener(glPageWrapper.VIEW_CHANGED, this._viewChanged, this);
    },

	/**
	* Cleanup memory usage
	* @function ?
	*/
	unload: function(){
	},

	/**
	* Try to listen for all available (and relevant) documents
	* @function {} ?
	*/
	_attachListeners: function(){
		if(!this._attachedListeners['main']){
			this._eventsContainer.observe(this._gtd.gmail.canvasDocument.body, 'contextmenu', this._context, true);
			this._eventsContainer.observe(this._gtd.gmail.canvasDocument.body, 'mousedown', this._click, false);
			this._attachedListeners['main'] = true;
		}
		if(!this._attachedListeners['contact']){
			if( this._gtd.gmail.canvasDocument.defaultView.top.frames[3].frames[0] ){
				var b = this._gtd.gmail.canvasDocument.defaultView.top.frames[3].frames[0].document.body;
				this._eventsContainer.observe(b, 'contextmenu', this._context, true);
				this._eventsContainer.observe(b, 'mousedown', this._click, false);
				this._attachedListeners['contact'] = true;
			}
		}
	},

	/**
	* Notification the gmail view changed
	* @function {} ?
	*/
	_viewChanged: function(evt){
		if( this._gtd.gmail.page.getActiveViewType()=="ct" ){
			setTimeout(glBase.bind(this, this._attachListeners), 2000);
			setTimeout(glBase.bind(this, this._attachListeners), 6000);
		}
	},
	
	/**
	* Respond to an environment click
	* @function ?
	*/
	_click: function(event){
		try{
			if( event.button!=2 ) return;
			var el = event.target;
			var details = this._getDetails(el);
			if( details ){
				glBase.stopEvent(event);
		
				// Get the popup
				var explorerPopup = this._gtd.getUIObject("explorerPopup");
				
				if( !details.specificCmd ){
					// Set the popup to use ThreadsViewer and grab a reference to it
					var threadsViewer = explorerPopup.getThreadsViewer(true);
					// Get the x/y below the Dashboard button:
					var xy = details.xy || explorerPopup.getXYFromElement(el, "bottom", true);
					xy = explorerPopup.boundsCheckXY(xy[0], xy[1], null, true);
					explorerPopup.active(true, xy[0], xy[1]);
					explorerPopup.setServerActionType(glActionManager.USE_AJAX);
					// Load into the ThreadsViewer
					threadsViewer.load( details.type, details.item, this._gtd.statusThreadsContainer );
				}else if( details.specificCmd=='loadTlThread' ){
					var gmailThreadListView = this._gtd.gmail.getCurrentViewUIObject();
					// Uncheck all checkboxes
					this._lastCheckedRows = gmailThreadListView.checkedRows();
					gmailThreadListView.uncheckRows();
			
					// Retrieve associated checkbox
					while( el.parentNode && el.nodeName!="TR" ) el = el.parentNode;
					el = el.getElementsByTagName("INPUT")[0];
					if( el.checked==false ) glUIWidget.simulateMouseEvent(el, "click");	// Check it
					// Get the Thread:
					var thread = gmailThreadListView.getThread( gmailThreadListView.getOffset(el) );
					this._currentThread = thread;
					// Set the popup to use ThreadsViewer and grab a reference to it
					var threadViewer = explorerPopup.getThreadViewer(true, null, null, false);
					// Get precise XY:
					var xy = details.xy || explorerPopup.getXYFromEvent(event, "", true);
					xy = explorerPopup.boundsCheckXY(xy[0], xy[1], null, true);
					// Reposition:
					explorerPopup.active(true, xy[0], xy[1]);
					explorerPopup.setServerActionType(glActionManager.USE_DOM);
					// Load into the ThreadViewer
					threadViewer.load( null, thread );	
					
					
					// Listen for thread changes
			    	this._gtd.gmail.actions.addEventListener(glActionManager.ARCHIVE.eventName, this._threadArchived, this);
					
					// Need to uncheck when closed
					explorerPopup.addEventListener(gtdExplorerPopup.HIDE, this._closeThreadPopup, this);
				}
		
			}
		}catch(e){
			glLogger.warn(e, 'gtdEnvironment._click', this._gtd.gmail);
		}
	},
	
	
	/**
	* Respond to an environment context click
	* @function ?
	*/
	_context: function(event){
		try{
			var el = event.target;
			
			var el2 = el;
			var i = 10;
			while( --i>=0 && el2.parentNode ){			
				if( el2.className=='lightbox' || el2.id=='gtdi-dash-container' ){
					glBase.stopEvent(event);
					return;
				}
				el2 = el2.parentNode;
			}
			
			var details = this._getDetails(el);
			if( details ) glBase.stopEvent(event);
		}catch(e){
			glLogger.warn(e, 'gtdEnvironment._context', this._gtd.gmail);
		}
	},
	
	
	/**
	* Extract details - if any - about what was clicked
	* @function {Object} ?
	* @param {Element} el - The el clicked
	* @return {Object} The details. Format: {type:String, item:String, xy:[], specificCmd:String}. Or null if no match.
	*/
	_getDetails: function(el){
		// Test for Contact in Gmail Contact List
		if( el.nodeName=="SPAN" && (el.className=="vs" || el.className=="HHshnc") ){
			return {type:gtdExplorerThreadsViewer.CONTACT, item: el.innerHTML};
		}
		
		// Test for Contact in Gmail Conversation
		// Test for Contact in Gmail ThreadList
		if( el.getAttribute("email") ){
			return {type:gtdExplorerThreadsViewer.CONTACT, item: el.getAttribute("email")};
		}
		
		
		// Test for Label in Gmail Conversation
		if( el.nodeName=="DIV" && el.className=="av" ){
			return {type:gtdExplorerThreadsViewer[this._gtd.labelsData.getLabelType(el.innerHTML).toUpperCase()], item:el.innerHTML};
		}
		
		// Test for Label in Gmail ThreadList
		if( el.nodeName=="SPAN" && el.className=="hN" ){
			return {type:gtdExplorerThreadsViewer[this._gtd.labelsData.getLabelType(el.innerHTML).toUpperCase()], item:el.innerHTML};
		}
		
		// Test for Label in Gmail Labels Box
		if( el.nodeName=="A" ){
			var i = 5;
			var el2 = el;
			while(--i>=0 && el2.parentNode){
				if( el2.getAttribute("gtdLabelName") && !el2.getAttribute("gtdi-placeholder") ){
					return {type:gtdExplorerThreadsViewer[this._gtd.labelsData.getLabelType(el2.getAttribute("gtdLabelName")).toUpperCase()], item:el2.getAttribute("gtdLabelName")};
				}else{
					el2 = el2.parentNode;
				}
			}
		}
		
		// Test for Contact in Gmail Contacts List
		if( el.className=='text' && glBase.testEmailAddress(el.title) ){
			return {type:gtdExplorerThreadsViewer.CONTACT, item: el.title};
		}
		
		// Test Email Link in Gmail Contacts List
		if( el.nodeName=='A' && glBase.testEmailAddress(el.innerHTML) ){
			return {type:gtdExplorerThreadsViewer.CONTACT, item: el.innerHTML};
		}		
		
		
		// Test for Row in Gmail ThreadList
		if( this._gtd.gmail.page.getActiveViewType()=="tl" ){
			if( !(el.className=="gtd-row-button" || el.className=="gtd-row-bin-button") ){	// Ignore button clicks
				var i = 10;
				while(--i>=0 && el.parentNode){
					if( el.nodeName=="TR" && el.className.indexOf('zA')>-1 ){
						return {specificCmd: 'loadTlThread'};
					}else{
						el = el.parentNode;
					}
				}
			}
		}
		
	},
	
	/**
	* Notification that a thread changed, from glActionManager
	* @function {} ?
	*/
	_threadArchived: function(evt){
			if( this._currentThread && this._currentThread.hasId(evt.item) && evt.environmentGenerated ){
					var explorerPopup = this._gtd.getUIObject("explorerPopup");
					explorerPopup.active(false);
			}
	},
	
	/**
	* Notification that the thread popup has closed.
	* Remove listeners and uncheck
	*/
	_closeThreadPopup: function(){
		
		var explorerPopup = this._gtd.getUIObject("explorerPopup");
		explorerPopup.removeEventListener(gtdExplorerPopup.HIDE, this._closeThreadPopup, this);
		
		// Restore checked rows:
		var gmailThreadListView = this._gtd.gmail.getCurrentViewUIObject();
		gmailThreadListView.checkedRows(this._lastCheckedRows);
		
    	this._gtd.gmail.actions.removeEventListener(glActionManager.ARCHIVE.eventName, this._threadArchived, this);
		this._currentThread = null;
	},

    //}
});
