/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


var gtdRecent = glBase.extend({
	// Properties {
	
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	// }
	
	/**
	* @constructor ?
	* @param {gtdMainPagePlugin} gtd - The main gtd plugin for this page
	*/
	constructor: function(gtd){
		this.base(gtd.gmail);
		this._gtd = gtd;
		
	        this._gtd.gmail.actions.addEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
	},
	
	/**
	* Callback for the {@link glActionManager.THREAD_CHANGED} event. Updates all
	* stored threads from the event and removes any that no longer fit the search
	* criteria.
	* @function ?
	* @param {Event} evt - The thread changed event
	*/
	_threadChanged: function(evt){
		if(evt.action === glActionManager.ADD_LABEL){
			var labelname = evt.label;
			
			// Extract type:
			var foundType = "";
			var typeSettings = this._gtd.accountData.getPref('labels.types');
			for( t in typeSettings ){
				if( gtdLabelsData.testPrefix(labelname, typeSettings[t].prefix) ){
					foundType = t;
					break;
				}
			}
			if( !foundType ) foundType = "misc";
			
			// Update Recent list:
			var recent = this._gtd.accountData.getPref('recent', true) || {};
			if( !recent[foundType] ) recent[foundType] = [];
			
			// Remove previous occurences:
			for( var i = 0; i < recent[foundType].length; i++ ){
				if( recent[foundType][i][0]==labelname ){
					recent[foundType].splice(i,1);
					break;
				}
			}
			// Add new label at front:
			recent[foundType].splice(0, 0, [labelname, (new Date()).valueOf()]);
			// Cap length
			if( recent[foundType].length > 5 ) recent[foundType].length = 5;
			
			this._gtd.accountData.setPref('recent', recent);
			this._gtd.accountData.savePrefs();
		}
	},
	
	/**
	* Get array of recent labels for a given type
	* @function {String[][]} ?
	* @param {String} type - The type of label to retrieve
	* @return Array of recent labels. Format: [[<labelname>, <dateadded>],]
	*/
	getLabels: function(type){
		var recent = this._gtd.accountData.getPref('recent', true) || {};
		if( recent[type] ){
			return recent[type];
		}else{
			return [];
		}
	},
	
	
});