/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Extends glSearch and adds GTD specific search helpers to it, like a tasks
 * method which restricts the search to tasks.
 *
 * @class gtdSearch @extends glSearch
 */
var gtdSearch = glSearch.extend({
    // Properties {
    /**
     * The gtdMainPagePlugin object for the page
     * @property {gtdMainPagePlugin} ?
     */
    _gtd: null,
    //}

    /**
     * The constructor.
     * @constructor ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    constructor: function(gtd) {
        this.base(gtd.gmail);
        this._gtd = gtd;
    },

    // Methods {
    /**
     * Adds a condition that restricts the search to tasks
     * @function {glSearch} ?
     * @return The object, so that it can be chained
     */
    tasks: function() {
        var condition = new glSearchOrConditions();
        condition.addCondition(glSearch.l('!Action'));
        condition.addCondition(glSearch.l('!Next Action'));
        condition.addCondition(glSearch.l('!Some Day'));
        condition.addCondition(glSearch.l('!Waiting On'));
        return this.addCondition(condition);
    },
    //}
});
