/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Add Tabs to Compose view when connected
 *
 * @class ? @extends gtdUIObject
 */
var gtdComposeTabs = gtdUIObject.extend({
    // Properties {
    
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
    
	/** @property {private Array} ? An array of els hidden after a tab click */
	_elsHidden: null,
	
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {Element} ? The tab container */
	el: null,
	
	/** @property {Element} ? The tab container for tab items */
	elTabs: null,
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        glLogger.log('Connecting gtdComposeTabs object to view', 'gtdComposeTabs');

        this.base(gtd);

	this._gtd = gtd;
	this._elsHidden = [];
	this._eventsContainer = new glEventsContainer(this);

	var doc = gtd.gmail.canvasDocument;
	
	this.elTabs = doc.createElement("DIV");
		this.elTabs.className = "compose-tabs";
	this.elTabs.innerHTML = "<div id='compose-tab-mail' class='compose-tab gtdi-active'>" + glI18N.getString("Compose.Mail") + "</div><div id='compose-tab-personal' class='compose-tab'>" + glI18N.getString("Compose.Personal") + "</div>";
	
	this.el = doc.createElement("TR");
	var elTd = doc.createElement("TD");
		elTd.setAttribute('colspan', '3');
	elTd.appendChild( this.elTabs );
	this.el.appendChild( elTd );
	
	var composeView = gtd.gmail.getCurrentViewUIObject();
	var elTBody = composeView.composeTable.getElementsByTagName("TBODY")[0];
	elTBody.insertBefore( this.el, elTBody.firstChild );
	
	this._eventsContainer.observe( this.el, "mousedown", this._click, false );

        //}
    },
    
	/**
	* Called when user clicks on tabs
	* @function ?
	*/
	_click: function(event){
		try{
			// Get the tab item clicked
			var elTab = event.target;
			
			this.select( elTab.id );
		}catch(e){
			glLogger.warn(e, 'gtdComposeTabs._click', this._gtd.gmail);
		}
	},
	
	/**
	* Select the tab with the specified id
	* @function ?
	* @param {String} id - The ID of the tab-item element to select
	*/
	select: function(id){
		
		var doc = this._gtd.gmail.canvasDocument;
		var elTab = doc.getElementById(id);
		if( !elTab ) return;

		// Remove anything selected
		for( var i = 0; i < this.elTabs.childNodes.length; i++ ){
			if( this.elTabs.childNodes[i].id==id && glUIWidget.hasClassName(this.elTabs.childNodes[i], 'gtdi-active') ) return;	// Currently selected right one; do nothing
			glUIWidget.removeClassName( this.elTabs.childNodes[i], "gtdi-active" );
		}
		
		// Restore view to normal (anything hidden)
		for( var i = 0; i < this._elsHidden.length; i++ ) this._elsHidden[i].style.display = "";
		this._elsHidden.length = 0;
		
		
		// Now update the compose table based on what was clicked
		var composeView = this._gtd.gmail.getCurrentViewUIObject();
		
		
		switch( elTab.id ){
			case "compose-tab-mail":
				composeView.toField.value = this._lastToFieldValue || "";
				break;
			case "compose-tab-personal":	// Set and hide To/From field
				var elTable = composeView.composeTable;
				var elRows = elTable.getElementsByTagName("TR");
				this._lastToFieldValue = composeView.toField.value;
				composeView.toField.value = this._gtd.gmail.environment.getAccount().replace("@", "+action@");
				var elSubject = null;
				for( var i = 1; i <= 6; i++ ){
					var elTrInput = elRows[i].getElementsByTagName("INPUT")[0];
					if( elTrInput && elTrInput.getAttribute("name")=="subject" ){
						elSubject = elTrInput;
						break; // Stop before Subject box. Can be located in alternate places depending on number whether user has multiple accounts.
					}
					if( elRows[i].style.display!="none" ){
						this._elsHidden.push( elRows[i] );
						elRows[i].style.display = "none";
					}
				}
				if( elSubject ){
					glUIWidget.simulateMouseEvent(elSubject, 'click');
					setTimeout(function(){
						glUIWidget.simulateMouseEvent(elSubject, 'click');
						elSubject.focus();
					},200);
				}
				break;
		}
		
		// Now mark current choice selected
		glUIWidget.addClassName( elTab, "gtdi-active");
	},
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting gtdComposeTabs from view', 'gtdComposeTabs');

        
        if( this.el.parentNode ) this.el.parentNode.removeChild( this.el );
        this.el = null;
        this.elTabs = null;
        this._eventsContainer.reset();
        
	// Restore view to normal (anything hidden)
	for( var i = 0; i < this._elsHidden.length; i++ ) this._elsHidden[i].style.display = "";
	this._elsHidden.length = 0;
        this.base();
    },
    
    /**
     * Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return false;
    },
    
    //}
});