/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The container for the thread UI, which can move through multiple threads in a set
 *
 * @class ? @extends gtdUIObject
 */
var gtdExplorerThreadViewer = gtdUIObject.extend({
    // Properties {
    
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	 
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	_elTitle: null, 
	
	_elCurrent: null,
	
	_elHeader: null,
	
	_threadUI: null, 
	
	_threads: null,
	
	_thread: null,
	
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
	*/
	connect: function(gtd) {
		glLogger.log('Connecting gtdExplorerThreadsItem object to view', 'gtdExplorerThreadsItem');
		
		
		this._gtd = gtd;
		this._eventsContainer = new glEventsContainer(this);
		this._threads = [];
		this.base(gtd);
		
		// Create container
		var doc = gtd.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.id = "gtdi-dash";
		
		// Create header
		this._elHeader = doc.createElement("DIV");
		this._elHeader.id = "gtdi-header";
		this.el.appendChild( this._elHeader );
		
		// Create header / breadcrumb
		
		var elBreadcrumb = doc.createElement("DIV");
		elBreadcrumb.className = 'gtdi-breadcrumb';
		this._elHeader.appendChild( elBreadcrumb );
		
		var elBack = doc.createElement("DIV");
		elBack.className = "gtdi-back";
		elBack.innerHTML = "<a href=''>" + glI18N.getString("ThreadViewer.Back") + "</a>";
		elBreadcrumb.appendChild( elBack );
		
		this._elTitle = doc.createElement("H2");
		elBreadcrumb.appendChild( this._elTitle );
		
		// Create header/paging
		var elPaging = doc.createElement("DIV");
		elPaging.id = 'gtdi-paging';
		elPaging.className = 'gtdi-btngrp';
		this._elHeader.appendChild( elPaging );
		
		elBtnGrp = doc.createElement("DIV");
		elBtnGrp.className = "gtdi-btn-grp";
		elPaging.appendChild( elBtnGrp );
		
		elBtn = doc.createElement("DIV");
		elBtn.className = "gtdi-btn";
		elBtn.innerHTML = '<div class="gtdi-btn"><b class="gtdi-btn-t"></b><b class="gtdi-btn-t2"></b><b class="gtdi-btn-m"></b><b class="gtdi-btn-b2"></b><b class="gtdi-btn-b"></b><span class="gtdi-btn-label"><span class="gtdi-paging-prev">' + glI18N.getString('ThreadViewer.Paging.prev') + '</span></span></div>';
		elBtnGrp.appendChild( elBtn );
		
		elBtn = doc.createElement("DIV");
		elBtn.className = "gtdi-btn";
		elBtn.innerHTML = '<div class="gtdi-btn"><b class="gtdi-btn-t"></b><b class="gtdi-btn-t2"></b><b class="gtdi-btn-m"></b><b class="gtdi-btn-b2"></b><b class="gtdi-btn-b"></b><span class="gtdi-btn-label"><span class="gtdi-paging-current"></span></span></div>';
		elBtnGrp.appendChild( elBtn );
		this._elCurrent = doc.evaluate(".//span[@class='gtdi-paging-current']", elBtn, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
		
		elBtn = doc.createElement("DIV");
		elBtn.className = "gtdi-btn";
		elBtn.innerHTML = '<div class="gtdi-btn"><b class="gtdi-btn-t"></b><b class="gtdi-btn-t2"></b><b class="gtdi-btn-m"></b><b class="gtdi-btn-b2"></b><b class="gtdi-btn-b"></b><span class="gtdi-btn-label"><span class="gtdi-paging-next">' + glI18N.getString('ThreadViewer.Paging.next') + '</span></span></div>';
		elBtnGrp.appendChild( elBtn );
		
		
		// Create body
		var elBody = doc.createElement("DIV");
		elBody.id = "gtdi-body";
		this.el.appendChild( elBody );
		
		this._threadUI = this._gtd.getUIObject('thread');
		elBody.appendChild( this._threadUI.el );
		
		
		// Add click handlers:
		this._eventsContainer.observe(this.el, 'mousedown', this._click, false);
		this._threadUI.addEventListener(gtdThread.CLICK_OPEN, this._clickOpen, this);
		
		// Initiate header:
		this.showHeader(true);
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		glLogger.log('Disconnecting gtdExplorerThreadsItem from view', 'gtdExplorerThreadsItem');
		
		if( this.el ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ){
			if( this[p] && this[p].nodeName && this[p].nodeType ){
				this[p] = null;
			}
		}
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		this._threads = null;
		this._thread = null;
		
		this.base();
	},
	
	/**
	* Specify the thread set to show
	* @function ?
	* @param {String} title - The title of this thread set
	* @param {glThread} thread - The thread to start on
	* @param {optional glThread[]} threads - The set of threads
	*/
	load: function(title, thread, threads){
		this._elTitle = title;
		this._threads = threads || [];
		this._loadThread(thread);
	},
	
	/**
	* Load an individual thread within a thread set
	* @function ?
	* @param {glThread} thread
	*/
	_loadThread: function(thread){
		this._thread = thread;
		this._threadUI.load(thread);
		this._setPaging();
	},
	
	/**
	* Update position (X of Y) on the paging UI
	* @function {} ?
	*/
	_setPaging: function(){
		for( var i = 0; i < this._threads.length; i++ ){
			if( this._threads[i].threadId==this._thread.threadId ){
				this._elCurrent.innerHTML = (i+1) + ' ' + glI18N.getString('ThreadViewer.Paging.of') + ' ' + this._threads.length;
				break;
			}
		}
	},
	
	/**
	* Move to the next available thread in the thread set
	* @function {} ?
	*/
	_pagingNext: function(){
		for( var i = 0; i < this._threads.length; i++ ){
			if( this._threads[i].threadId==this._thread.threadId ){
				if( this._threads[i+1] ){
					this._loadThread(this._threads[i+1]);
					break;
				}
			}
		}
	},
	
	/**
	* Move to the previous available thread in the thread set
	* @function {} ?
	*/
	_pagingPrev: function(){
		for( var i = 0; i < this._threads.length; i++ ){
			if( this._threads[i].threadId==this._thread.threadId ){
				if( this._threads[i-1] ){
					this._loadThread(this._threads[i-1]);
					break;
				}
			}
		}
	},
	
	_click: function(event){
		try{
			var el = event.target;
		
			while( el.parentNode ){
				if( el.className=='gtdi-back' ){
					glBase.stopEvent(event);
					this.dispatchEvent({name: gtdExplorerThreadViewer.CLICK_BACK});
					break;
				}
				if( el.className=='gtdi-btn' ){
					// Clicked a button, see if it contains prev or next
					var elBtn = el.ownerDocument.evaluate(".//*[contains(@class,'gtdi-paging-prev') or contains(@class, 'gtdi-paging-next')]", el, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
					if( elBtn ){
						if( elBtn.className=='gtdi-paging-prev' ){
							glBase.stopEvent(event);
							this._pagingPrev();
							break;
						}
						if( elBtn.className=='gtdi-paging-next' ){
							glBase.stopEvent(event);
							this._pagingNext();
							break;
						}
					}
				}
				if( el.id=='gtdi-header' || el.className=='gtdi-thread-subject' ){
					glBase.stopEvent(event);
					this.dispatchEvent({name: gtdExplorerThreadViewer.CLICK_HEADER, mouseEvent:event});
					break;
				}
				el = el.parentNode;
			}
		}catch(e){
			glLogger.warn(e, 'gtdExplorerThreadViewer._click', this._gtd.gmail);
		}
	},
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type){
		// Propage to the thread
		this._threadUI.setServerActionType(type);
	},
	
	/**
	* Set the visibility of the header
	* @function ?
	* @param {Boolean} v - If true, make visible
	*/
	showHeader: function(v){
		this._elHeader.style.display = v? '' : 'none';
		if( v ){
			glUIWidget.removeClassName(this.el, 'gtdi-no-header');
			glUIWidget.addClassName(this.el, 'gtdi-has-header');
		}else{
			glUIWidget.removeClassName(this.el, 'gtdi-has-header');
			glUIWidget.addClassName(this.el, 'gtdi-no-header');
		}
	},
	
	/**
	* Respond to user clicking 'Open Mail'
	* @function {} ?
	*/
	_clickOpen: function(){
		this.dispatchEvent({name: gtdExplorerThreadViewer.CLICK_OPEN});
	},
	
	/**
	* Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
	* @function {Boolean} ?
	* @return Whether the UI can be cached
	*/
	isCacheable: function() {
		return false;
	},
	

},{
	CLICK_BACK: 'clickedBack',
	CLICK_HEADER: 'clickHeader',
	CLICK_OPEN: 'clickOpen',
});
gtdExplorerThreadViewer.implement(glEventDispatcher); // Add event dispatching support