/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The UI for switching view buttons
 *
 * @class ? @extends gtdUIObject
 */
var gtdExplorerViewSwitch = gtdUIObject.extend({
    // Properties {
    
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	 
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	
	
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
	*/
	connect: function(gtd) {
		glLogger.log('Connecting gtdExplorerThreadsItem object to view', 'gtdExplorerThreadsItem');
		
		this._gtd = gtd;
		this._eventsContainer = new glEventsContainer(this);
		this.base(gtd);
		
		// Create container
		var doc = gtd.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.id = "gtdi-viewswitch";
		
		var elUl = doc.createElement("UL");
		
		var views = [gtdExplorerViewSwitch.LIST_VIEW, gtdExplorerViewSwitch.PROJECT_VIEW];
		
		for( var i = 0; i < views.length; i++ ){
			var elLi = doc.createElement("LI");
			elLi.innerHTML = "<a gtdi-viewswitch='" + views[i] + "' id='gtdi-viewswitch-" + views[i] + "' href='#'>" + glI18N.getString('ViewSwitch.' + views[i]) + "</a>";
			elLi.firstChild.title = glI18N.getString('ViewSwitch.' + views[i]);
			elUl.appendChild( elLi );
		}
		
		this.el.appendChild( elUl );
		this._eventsContainer.observe( this.el, 'click', this._click, false );
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		glLogger.log('Disconnecting gtdExplorerThreadsItem from view', 'gtdExplorerThreadsItem');
		
		if( this.el ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ){
			if( this[p] && this[p].nodeName && this[p].nodeType ){
				this[p] = null;
			}
		}
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		this.base();
	},
	
	
	/**
	* When any element is clicked; figure out the selection, update the active status, 
	* @function ?
	*/
	_click: function(event){
		try{
			var el = event.target;
			var id = el.getAttribute('gtdi-viewswitch');
			this.setView(id);
		}catch(e){
			glLogger.warn(e, 'gtdExplorerViewSwitch._click', this._gtd.gmail);
		}
	},
	
	/**
	* Set the active button and send notification of change
	* @function ?
	* @param {String} id - The id of the view to activate. Static property.
	*/
	setView: function(id){
		
		var doc = this._gtd.gmail.canvasDocument;
		var els = doc.evaluate( ".//a[@gtdi-viewswitch]", this.el, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
		for( var i = 0; i < els.snapshotLength; i++){
			if( els.snapshotItem(i).getAttribute('gtdi-viewswitch')==id ){
				glUIWidget.addClassName(els.snapshotItem(i), 'gtdi-active');
			}else{
				glUIWidget.removeClassName(els.snapshotItem(i), 'gtdi-active');
			}
		}
		
		this.dispatchEvent({name: gtdExplorerViewSwitch.CHANGE_VIEW, view: id});
	},
	
	/**
	* Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
	* @function {Boolean} ?
	* @return Whether the UI can be cached
	*/
	isCacheable: function() {
		return false;
	},
	

}, {
	CHANGE_VIEW: 'changeView',
	
	LIST_VIEW: 'list',
	PROJECT_VIEW: 'project',
});
gtdExplorerViewSwitch.implement(glEventDispatcher); // Add event dispatching support