/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * A combo-box for a type of labels. Belongs to gtdLabellingBox.
 *
 * @class ? @extends gtdUIObject
 */
var gtdLabellingDropdown = gtdUIObject.extend({
    // Properties {
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {private gtdLabellingBox} ? The Labelling box that owns this dropdown */
	_labellingBox: null,
	
	/** @property {Element} ? The label container */
	el: null,
	
	/** @property {Element} ? The SPAN title element */
	_elTitle: null,
	
	/** @property {Element} ? The dropdown */
	_elDropdown: null,
	
	/** @property {Element} ? List of recent labels */
	_elRecent: null,
	
	/** @property {Element} ? Title of recent labels */
	_elRecentTile: null,
	
	/** @property {string} ? The type of labels this contains (e.g. 'statuses') */
	_type: null,
	
	/** @property {gtdLabelDOM} ? Collection of gtdLabelDOM objects and container el for labels.  */
	_typeLabelDOM: null,
	
    	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
    //}

	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
	*/
	connect: function(gtd) {
		glLogger.log('Connecting gtdLabellingDropdown object to view', 'gtdLabellingDropdown');
		
		this.base(gtd);
		this._gtd = gtd;
		this._eventsContainer = new glEventsContainer(this);
		
			
		// Create the nodes:
		var doc = gtd.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = "gtdi-btn gtdi-dropdown";
		this.el.innerHTML = "<b class='gtdi-btn-t'></b><b class='gtdi-btn-t2'></b><b class='gtdi-btn-m'></b><b class='gtdi-btn-b2'></b><b class='gtdi-btn-b'></b><span class='gtdi-btn-label'></span>";
		this._elTitle = this.el.getElementsByTagName("SPAN")[0];
		
		
		// Create the dropdown nodes:
		this._elDropdown = doc.createElement("DIV");
		this._elDropdown.className = "gtdi-droppeddown";
		this._elDropdown.style.display = "none";
		this.el.appendChild( this._elDropdown );
	},
	
	/**
	* Load the labelling box
	* @function ?
	* @param {string} type - The type of labels this contains (E.g. 'statuses')
	*/
	setType: function(type){
		this._type = type;
	
	
		this.refreshTitle();
		
		
		this._elDropdown.innerHTML = "";
		
		var doc = this._gtd.gmail.canvasDocument;
		// Create recent:
		this._elRecentTitle = doc.createElement("DIV");
		this._elRecentTitle.className = 'gtdi-droppeddown-recent-title';
		this._elRecentTitle.innerHTML = glI18N.getString("Labelling.dropdown.recent");
		this._elDropdown.appendChild( this._elRecentTitle );
		
		this._elRecent = doc.createElement("DIV");
		this._elRecent.className = 'gtdi-droppeddown-recent';
		this._elDropdown.appendChild(this._elRecent);
		
		
		// Create the labels box:
		this._typeLabelDOM = this._gtd.labelsData.getLabelsEl(this._type);
		this._elDropdown.appendChild( this._typeLabelDOM.el );
		
		// Listen for button clicks
		this._clickWindow();
	},
	
	_clickButton: function(event){
		try{
			var doc = this.el.ownerDocument;
			// Render recent
			this._elRecent.innerHTML = "";
			var recent = this._gtd.recent.getLabels(this._type);
			if( recent.length==0 ){
				this._elRecentTitle.style.display = "none";
			}else{
				this._elRecentTitle.style.display = "";
				for( var i = 0; i < recent.length; i++ ){
					var elRecent = doc.createElement("DIV");
					elRecent.className = "gtdi-droppeddown-recent-item";
					elRecent.innerHTML = recent[i][0];
					elRecent.setAttribute("gtdLabelName", recent[i][0]);
					this._elRecent.appendChild( elRecent );
				}
			}
		
			// Close all parent nodes and their children
			this._typeLabelDOM.close();
			//for( l in this._typeLabelDOM.labelHash ) if( this._typeBuckets.labels[l].depth==0 ) this._typeBuckets.labels[l].closeLabel();
		
			// Open all selected:
			var labels = this._typeLabelDOM.getSelectedLabelNames();
			for( var i = 0; i < labels.length; i++ ) this._typeLabelDOM.open( labels[i] );
			//for( l in this._typeBuckets.labels ) if( this._typeBuckets.labels[l].selected() ) this._typeBuckets.labels[l].openLabelsToHere();
		
			// Show it and listen for clicks that will close it
			this._eventsContainer.stopObserving( this.el, "click", this._clickButton, false );
			setTimeout( glBase.bind(this, function(){
				this._eventsContainer.observe( this.el.ownerDocument.body, "mousedown", this._clickWindow, true );
				this._elDropdown.style.display = "";
			}), 10);
		}catch(e){
			glLogger.warn(e, 'gtdLabellingDropdown._clickButton', this._gtd.gmail);
		}
	},
	
	_clickWindow: function(event){
		try{
			// Check not clicked expander
			if( event ){
				var el = event.target;
				if( glUIWidget.hasClassName(el, 'gtdi-labeldom-expander') ){
					glBase.stopEvent(event);
					return;
				}
			}
			
			// Hide it and listen for clicks that will open it
				this._eventsContainer.stopObserving( this.el.ownerDocument.body, "mousedown", this._clickWindow, true );
				setTimeout( glBase.bind( this, function(){
					this._eventsContainer.observe( this.el, "click", this._clickButton, false );
					this._elDropdown.style.display = "none";		
				}), 250 );	
	
		
		}catch(e){
			glLogger.warn(e, 'gtdLabellingDropdown._clickWindow', this._gtd.gmail);
		}
	},
	
	refreshTitle: function(){
		// Get a count of active labels
		var activeLabelCount = 0;
		
		// Update the title element
		this._elTitle.innerHTML = glI18N.getString("Labelling.dropdown." + this._type + ".title", glI18N.getString("Labels.Types." + this._type, this._type));
		if( activeLabelCount > 0 ) this._elTitle.innerHTML += " (" + activeLabelCount + ")";
	},
	
	
	
	/**
	* Visually set a label as selected or unselected 
	* @function ?
	* @param {string} labelname - The name of the label to update
	* @param {boolean} isOn - If true, mark selected. Otherwise, mark unselected.
	*/
	setLabel: function(labelname, isOn){
		this._typeLabelDOM.selected(labelname, isOn);
		//if( this._typeBuckets.labels[labelname] ) this._typeBuckets.labels[labelname].select(isOn);
	},
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting gtdLabellingDropdown from view', 'gtdLabellingDropdown');

        
        this._gtd = null;
        if( this.el.parentNode ) this.el.parentNode.removeChild( this.el );
        this.el = null;
        this._elTitle = null;
        this._eventsContainer.reset();
        this._eventsContainer = null;
        this.base();
    },
    
    /**
     * Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return false;
    },
    
    //}
},{
    // Static Properties {
    /**
     * Dispatched when a label is selected
     * @event ?
     */
    SELECT_LABEL: 'select_label',
    
    /**
     * Dispatched when a label is unselected
     * @event ?
     */
    UNSELECT_LABEL: 'unselect_label',
});
gtdLabellingDropdown.implement(glEventDispatcher); // Add event dispatching support