/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Loads all the GTD widgets that can be activated in the general view based on
 * user preferences, such as {@link gtdSearchFiltersDropdown}.
 *
 * @class ? @extends gtdUIObject
 */
var gtdMainView = gtdUIObject.extend({
    // Properties {
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
    	try{
	        glLogger.log('Connecting gtdMainView object to view', 'gtdMainView');
	
	        this.base(gtd);
	
	        if(gtd.accountData.getPref('components.compose_personal.enabled')) {
	        	gtd.getUIObject('composePersonal');
	        }
	
	        if(gtd.accountData.getPref('components.search_button.enabled')) {
	        	gtd.getUIObject('searchButton');
	        }
	
			gtd.getUIObject('navBar');
	
	
	        glLogger.log('Connected gtdMainView object to view', 'gtdMainView');
	}catch(e){
		glLogger.error('Error constructing gtdMainView\n' + e.toString(), 'gtdMainView');
	}
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting gtdMainView from view', 'gtdMainView');

        this.base();
    },
    //}
});