/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Tweak the Nav bar
 *
 * @class ? @extends gtdUIObject
 */
var gtdNavBar = gtdUIObject.extend({
    // Properties {
    /** @property {glNavBar} ? The abstraction layer for the navigation menu */
    _glNavBar: null,
	
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {Element} ? The dashboard button */
	_elDashboard: null,
	
	/** @property {Element} ? The count element within the dashboard button */
	_elDashboardCount: null,

    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        glLogger.log('Connecting gtdNavBar object to view', 'glNavBar');

        this.base(gtd);
		this._gtd = gtd;
        this._eventsContainer = new glEventsContainer(this);

		var doc = gtd.gmail.canvasDocument;
		this._glNavBar = gtd.gmail.getUIObject("navBar");
		
		// Add the Dashboard:
		
		
        if(gtd.accountData.getPref('components.dashboard.enabled')) {
			var elSpam = this._glNavBar.getNavItem(glNavBar.SPAM);
			
			/*
			this._elDashboard = doc.createElement("DIV");
			this._elDashboard.className = 'gtdi-navitem gtdi-navitem-dashboard';
			this._elDashboard.className += ' qi pX';	// Add standard Gmail classes
			this._elDashboard.innerHTML = glI18N.getString("Dashboard.Button");
			
			this._elDashboardCount = doc.createElement("SPAN");
			this._elDashboard.appendChild( this._elDashboardCount );
			*/
			if( elSpam.nodeName=="TR" ){
				// New type of Gmail
				this._elDashboard = doc.createElement("TR");
				this._elDashboard.className = 'Alfa2e';
				var elTD = doc.createElement("TD");
				elTD.className = "nL";
				this._elDashboard.appendChild( elTD );
				var elTD = doc.createElement("TD");
				elTD.className = "n2 n1";
				this._elDashboard.appendChild( elTD );
				elTD.innerHTML = "<div class='n2'><div class='nU'><a class='n0' gtdi-navlink='1' target='_top' href='#'>" + glI18N.getString("Dashboard.Button") + "</a></div></div>";
			}else{
				// Old type of Gmail
				this._elDashboard = doc.createElement("DIV");
				this._elDashboard.className = 'qi pX';
				this._elDashboard.innerHTML = "<div class='nH'><div class='nH'><div style='height: 4px;' class='pi'><div style='width: 4px;' class='ph p'></div><div style='width: 4px;' class='ph q'></div><div class='l o'></div></div><div class='l m'><div style='margin-left: 4px;' class='l n'><div style='margin: -2px 4px -2px 0px; padding: 0px;' class='k'><div class='diLZtc'><div class='nH'><div class='qk'><span idlink='' class='' id=':q7'><div style='overflow: hidden; white-space: nowrap;'><div idlink='' class='p5' id=':qo'><a gtdi-navlink='1' class='qh' target='_top' href='#'>" + glI18N.getString("Dashboard.Button") + "</a></div></div></span></div></div></div></div></div></div><div class='dJ'></div><div style='height: 4px;' class='pi'><div style='width: 4px;' class='ph i'></div><div style='width: 4px;' class='ph j'></div><div class='l h'></div></div></div></div>";
			}
			
			if( elSpam.nextSibling && elSpam.nextSibling.nextSibling ){
				elSpam.parentNode.insertBefore( this._elDashboard, elSpam.nextSibling.nextSibling );
			}else{
				elSpam.parentNode.appendChild( this._elDashboard );
			}
			
			this._elDashboardCount = doc.evaluate('.//a[@gtdi-navlink]', elSpam.parentNode, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
			
    		this._eventsContainer.observe( this._elDashboard, "mousedown", this._clickDashboard, false );
			this._refreshDashboardCount();
			this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.LOADED, this._refreshDashboardCount, this);
			this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.LOADING_UPDATE, this._refreshDashboardCount, this);
			this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.CHANGED, this._refreshDashboardCount, this);
        }
		
        
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting gtdNavBar from view', 'glNavBar');
        this._glNavBar = null;
		
        this._eventsContainer.reset();
        
        if( this._elDashboard && this._elDashboard.parentNode ) this._elDashboard.parentNode.removeChild( this._elDashboard );
        this._elDashboard = null;

        this.base();
    },

	
    /**
    * Handle a button click
    * @function ?
    */
    _clickDashboard: function(event){
		try{
			glBase.stopEvent(event);
			// Get the popup
			var explorerPopup = this._gtd.getUIObject("explorerPopup");
			

			// Set the popup to use ThreadsViewer and grab a reference to it
			var threadsViewer = explorerPopup.getThreadsViewer(true);
			
			// Get the x/y below the Dashboard button:
			var xy = explorerPopup.getXYFromElement(this._elDashboard, "right", true);
			xy = explorerPopup.boundsCheckXY(xy[0], xy[1], null, true);
			explorerPopup.active(true, xy[0], xy[1]);
			explorerPopup.setServerActionType(glActionManager.USE_AJAX);
			
			// Load the Dashboard into the ThreadsViewer
			threadsViewer.load( gtdExplorerThreadsViewer.DASHBOARD, null, this._gtd.statusThreadsContainer );
			
			return;
			//this._gtd.gmail.actions.menuActions.addStarToThread();
			//return;
			this._gtd.gmail.actions.menuActions.addLabelToThread('S/Action');
			this._gtd.gmail.actions.menuActions.addLabelToThread('S/Next Action');
			this._gtd.gmail.actions.menuActions.addLabelToThread('P/GTDInbox/Bugs');
			this._gtd.gmail.actions.menuActions.addStarToThread();
			this._gtd.gmail.actions.menuActions.markThreadRead();
			this._gtd.gmail.actions.menuActions.removeStarFromThread();
			this._gtd.gmail.actions.menuActions.archiveThread();
			
		}catch(e){
			glLogger.warn(e, "gtdNavBar._clickDashboard", this._gtd.gmail);
		}
    },

	_refreshDashboardCount: function(){
		if( this._gtd.statusThreadsContainer.threads.length==0 ){
			this._elDashboardCount.innerHTML = glI18N.getString("Dashboard.Button");
			this._elDashboard.style.fontWeight = 'normal';
		}else{
			this._elDashboardCount.innerHTML = glI18N.getString("Dashboard.Button") + " (" + this._gtd.statusThreadsContainer.threads.length + ")";
			this._elDashboard.style.fontWeight = 'bold';
		}
	},

    //}
});
