/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */



/**
 * A self-contained UI for a thread
 *
 * @class ? @extends gtdUIObject
 */
var gtdThread = gtdUIObject.extend({
    // Properties {
    
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
    	   	
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	_elSubject: null,
	
	_elMessages: null,
	
	/** @property {private glThread} ? The associated thread */
	thread: null,
	
	
	/** @property {private gtdLabellingBox} ? The labelling box */
	_labellingBox: null,
	
	
    //}


	// Methods {
	/**
	* @function ?
	* @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
	*/
	connect: function(gtd) {
		glLogger.log('Connecting gtdThread object to view', 'gtdThread');
		
		
		this._gtd = gtd;
		this._eventsContainer = new glEventsContainer(this);
		this.base(gtd);
		
		// Create container
		var doc = gtd.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.id = "gtdi-thread";
		
		// Subject and link:
		var elH1 = doc.createElement("H1");
		elH1.className = 'gtdi-thread-subject';
		this.el.appendChild( elH1 );
		
		this._elSubject = doc.createElement("SPAN");
		elH1.appendChild( this._elSubject );
		
		var elLink = doc.createElement("A");
		elLink.href = "#";
		elLink.id = 'gtdi-thread-open';
		elLink.innerHTML = glI18N.getString("Thread.Open");
		elH1.appendChild( elLink );
		
		
		// Labelling Box
		this._labellingBox = gtd.getUIObject('labellingBox');
		this.el.appendChild( this._labellingBox.el );
		
		
		// Messages container
		this._elMessages = doc.createElement("DIV");
		this._elMessages.id = "gtdi-messages";
		this.el.appendChild( this._elMessages );
		
		
		// Event handlers:
		this._eventsContainer.observe(this.el, "click", this._click, false);
		
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		glLogger.log('Disconnecting gtdThread from view', 'gtdThread');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ){
			if( this[p] && this[p].nodeName && this[p].nodeType ){
				this[p] = null;
			}
		}
		
		
		if( this.thread ) this.thread.removeEventListener(glThread.LOADED, this._threadLoaded, this);
		this.thread = null;
		
		
		this._labellingBox.disconnect();
		this._labellingBox = null;
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		
		this.base();
	},

	/**
	* Notification our thread object has been updated
	*/
	_threadLoaded: function(event){
		this.load(this.thread);
	},

	/**
	* Render a thread
	* @function ?
	* @param {glThead} thread - The thread to render
	*/
	load: function(thread){
		
		if( this.thread ){
			this.thread.removeEventListener(glThread.LOADED, this._threadLoaded, this);
		}
		
		this.thread = thread;
		this.thread.addEventListener(glThread.LOADED, this._threadLoaded, this);
		
		// Load basic details:
		this._elSubject.innerHTML = thread.subjectSnippet;
		this._labellingBox.load(thread, true);
		
		// Load the messages
		var doc = this.el.ownerDocument;
		this._elMessages.innerHTML = "";
		
		var messages = this.thread.getThreadDetails().messages;
		for( var i = 0; i < messages.length; i++ ){
			var elMessage = this._createMessageEl(messages[i]);
			if( i<(messages.length-1) ) elMessage.className += " gtdi-closed";
			else elMessage.className += " gtdi-final";
			
			this._elMessages.appendChild( elMessage );
		}
	},
	
	/**
	* Create an element for a given message
	* @function {Element} ?
	* @param {glThreadMessage} message
	* @return The message element
	*/
	_createMessageEl: function(message){
		// TODO recycle message elements
		
		var doc = this.el.ownerDocument;
		
		var elMessage = doc.createElement("DIV");
		elMessage.className = "gtdi-message";
		
		// Message Head
		var elMessageHead = doc.createElement("DIV");
		elMessageHead.className = 'gtdi-messagehead';
		elMessage.appendChild( elMessageHead );
		
		var elUl = doc.createElement("UL");
		elMessageHead.appendChild( elUl );
		
		var elMessageFrom = doc.createElement("LI");
		elMessageFrom.className = 'gtdi-messagefrom';
		elMessageFrom.innerHTML = "<span email='" + message.sender.email + "'>" + message.sender.name + "</span>";
		elUl.appendChild( elMessageFrom );
		
		var elMessageSnippet = doc.createElement("LI");
		elMessageSnippet.className = 'gtdi-messagesnippet';
		elMessageSnippet.innerHTML = "<span>" + message.message + "</span>";
		elUl.appendChild( elMessageSnippet );
		
		var elMessageDate = doc.createElement("LI");
		elMessageDate.className = 'gtdi-messagedatetime';
		elMessageDate.innerHTML = "<span>" + message.date + "</span>";
		elUl.appendChild( elMessageDate );
		
		// Message Body
		var elMessageBody = doc.createElement("DIV");
		elMessageBody.className = "gtdi-messagebody";
		elMessage.appendChild( elMessageBody );
		
		elMessageContent = doc.createElement("DIV");
		elMessageContent.className = "gtdi-messagecontent";
		elMessageContent.innerHTML = message.message;
		elMessageBody.appendChild( elMessageContent );
		
		return elMessage;
	},
	
	
	/**
	* Respond to a click of the messages
	* @function ?
	*/
	_click: function(event){
		try{
			var el = event.target;
			
			if( el.id=="gtdi-thread-open" ){
				// Load the current thread ID
				var win = this.el.ownerDocument.defaultView.top;
				var domain = win.location.href.match(/^http(s?):\/\/mail.google.com\/(a\/.+?\/|.+?\/)/)[0];
				var path = win.location.href;
				path = path.replace(domain, "");	// path now in the format #inbox/

				if( path.indexOf("#search")==0 || path.indexOf("#label")==0 ){	// It's in the double format, #search/searchstr
					var m = path.match( /(#.+?\/.+?)(\/[^\/]+$|$)/ );
					if( m ) path = m[1];
					else path = "";
				}else{
					var m = path.match( /(#.+?)(\/[^\/]+$|$)/ );
					if( m ) path = m[1];
					else path = "";
				}
				if( !path ) path = "#inbox";

				var url = domain + path + "/" + this.thread.threadId;
				win.location.href = url;
				
				this.dispatchEvent({name:gtdThread.CLICK_OPEN, thread: this.thread});
			}else{
				
				// Scan up tree to find if a header was clicked
				while( el.parentNode && !glUIWidget.hasClassName(el,"gtdi-message") ) el = el.parentNode;
				if( glUIWidget.hasClassName(el, "gtdi-message") ){
					// Clicked header, toggle
					if( glUIWidget.hasClassName(el, 'gtdi-closed') ){
						glUIWidget.removeClassName(el, 'gtdi-closed');
					}else{
						glUIWidget.addClassName(el, 'gtdi-closed');
					}
				}
				
			}
			
		}catch(e){
			glLogger.warn(e, "gtdThreadListContainer._clickFiltersBar", this._gtd.gmail);
		}
	},
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type){
		this._labellingBox.setServerActionType(type);
	},
	
	
	/**
	* Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
	* @function {Boolean} ?
	* @return Whether the UI can be cached
	*/
	isCacheable: function() {
		return false;
	},
	

},{
	CLICK_OPEN: 'clickOpen',
});
gtdThread.implement(glEventDispatcher); // Add event dispatching support